Module: Glimmer::Web::Component::GlimmerSupersedable

Defined in:
lib/glimmer/web/component.rb

Overview

This module was only created to prevent Glimmer from checking method_missing first

Instance Method Summary collapse

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(method_name, *args, &block) ⇒ Object



136
137
138
139
140
141
# File 'lib/glimmer/web/component.rb', line 136

def method_missing(method_name, *args, &block)
  # TODO think of a way to hide the unnecessary exception noise that is printed to the user in the browser console whenever this is triggered
  Glimmer::DSL::Engine.interpret(method_name, *args, &block)
rescue
  super(method_name, *args, &block)
end